/* ************************************************************************
 * Copyright 2006 Lucio Benfante
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *********************************************************************** */
package com.benfante.splasher.task;

import java.util.ArrayList;
import java.util.List;

/**
 * Base implementation for task classes.
 *
 * @author Lucio Benfante <a href="mailto:lucio@benfante.com">lucio@benfante.com</a>
 * @version $Id: AbstractTask.java,v 1.1.1.1 2006-04-01 05:41:35 lucio Exp $
 */
public abstract class AbstractTask implements Task {
    
    public final static int COMPLETED = 100;
    
    protected String description;
    protected int weight;
    
    /** The percent of completion of the task */
    protected int completed = 0;
    
    private List<TaskObserver> observers = new ArrayList<TaskObserver>();
    
    /**
     * Creates a new instance of AbstractTask.
     *
     * @param description A description for the task
     * @param weight The weight of this task (relative to the other tasks)
     */
    public AbstractTask(String description, int weight) {
        this.description = description;
        this.weight = weight;
    }
    
    /**
     * Add a task observer.
     *
     * @param observer The task observer that will be notified about changes
     *                 on this task.
     */
    public void add(TaskObserver observer) {
        observers.add(observer);
        observer.notifyAdd(this);
    }
    
    /**
     * Return the percent of completion of the task.
     *
     * @return The percent of completion of the task.
     */
    public int getCompleted() {
        return this.completed;
    }
    
    /**
     * Return the description of the task.
     *
     * @return The description of the task.
     */
    public String getDescription() {
        return this.description;
    }
    
    /**
     * Return the weight of the task.
     *
     * @return The weight of the task.
     */
    public int getWeight() {
        return this.weight;
    }
    
    /**
     * Check if the task is completed.
     *
     * @return true if the task is completed.
     */
    public boolean isCompleted() {
        return this.completed == COMPLETED;
    }
    
    /**
     * Notify changes to the task observer(s).
     */
    public void notifyChanges() {
        for(TaskObserver observer : observers) {
            observer.update(this);
        }
    }
    
}
